//
// Created by robin on 1/8/18.
//

#include "dns-base/QDnsPacket.h"

namespace QDns
{
    namespace Base
    {
        QDnsPacket::QDnsPacket()
                : m_isValid(true)
        {
        }

        QDnsPacket QDnsPacket::parse(const QByteArray &data)
        {
            QDataStream dataStream(data);
            return parse(data, dataStream);
        }

        QDnsPacket QDnsPacket::parse(const QByteArray &data, QDataStream &dataStream)
        {
            QDnsPacket packet;

            packet.m_header = QDnsHeader::parse(dataStream);
            if (!packet.m_header.isValid())
            {
                packet.m_isValid = false;
            }
            else
            {
                for (quint16 i = 0; i < packet.m_header.getQueryCount() && packet.m_isValid; ++i)
                {
                    auto entry = QDnsQuery::parse(data, dataStream);
                    if (!entry.isValid())
                    {
                        packet.m_isValid = false;
                    }
                    packet.m_queries.append(entry);
                }
                for (quint16 i = 0; i < packet.m_header.getAnswerCount() && packet.m_isValid; ++i)
                {
                    auto entry = QDnsResourceRecord::parse(data, dataStream);
                    if (!entry.isValid())
                    {
                        packet.m_isValid = false;
                    }
                    packet.m_answerRecords.append(entry);
                }
                for (quint16 i = 0; i < packet.m_header.getAuthorityCount() && packet.m_isValid; ++i)
                {
                    auto entry = QDnsResourceRecord::parse(data, dataStream);
                    if (!entry.isValid())
                    {
                        packet.m_isValid = false;
                    }
                    packet.m_authorityRecords.append(entry);
                }
                for (quint16 i = 0; i < packet.m_header.getAdditionalCount() && packet.m_isValid; ++i)
                {
                    auto entry = QDnsResourceRecord::parse(data, dataStream);
                    if (!entry.isValid())
                    {
                        packet.m_isValid = false;
                    }
                    packet.m_additionalRecords.append(entry);
                }
            }

            return packet;
        }

        QByteArray QDnsPacket::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsPacket::serialize(QDataStream &dataStream) const
        {
            m_header.serialize(dataStream);
            for (const auto &entry : m_queries)
            {
                entry.serialize(dataStream);
            }
            for (const auto &entry : m_answerRecords)
            {
                entry.serialize(dataStream);
            }
            for (const auto &entry : m_authorityRecords)
            {
                entry.serialize(dataStream);
            }
            for (const auto &entry : m_additionalRecords)
            {
                entry.serialize(dataStream);
            }
        }

        bool QDnsPacket::isValid() const
        {
            return m_isValid;
        }

        const QDnsHeader &QDnsPacket::getHeader() const
        {
            return m_header;
        }

        void QDnsPacket::setHeader(const QDnsHeader &header)
        {
            m_header = header;
            m_header.setQueryCount((quint16) m_queries.length());
            m_header.setAnswerCount((quint16) m_answerRecords.length());
            m_header.setAuthorityCount((quint16) m_authorityRecords.length());
            m_header.setAdditionalCount((quint16) m_additionalRecords.length());
        }

        const QList<QDnsQuery> &QDnsPacket::getQueries() const
        {
            return m_queries;
        }

        void QDnsPacket::setQueries(const QList<QDnsQuery> &queries)
        {
            m_queries = queries;
            m_header.setQueryCount((quint16) m_queries.length());
        }

        const QList<QDnsResourceRecord> &QDnsPacket::getAnswerRecords() const
        {
            return m_answerRecords;
        }

        void QDnsPacket::setAnswerRecords(const QList<QDnsResourceRecord> &answerRecords)
        {
            m_answerRecords = answerRecords;
            m_header.setAnswerCount((quint16) m_answerRecords.length());
        }

        const QList<QDnsResourceRecord> &QDnsPacket::getAuthorityRecords() const
        {
            return m_authorityRecords;
        }

        void QDnsPacket::setAuthorityRecords(const QList<QDnsResourceRecord> &authorityRecords)
        {
            m_authorityRecords = authorityRecords;
            m_header.setAuthorityCount((quint16) m_authorityRecords.length());
        }

        const QList<QDnsResourceRecord> &QDnsPacket::getAdditionalRecords() const
        {
            return m_additionalRecords;
        }

        void QDnsPacket::setAdditionalRecords(const QList<QDnsResourceRecord> &additionalRecords)
        {
            m_additionalRecords = additionalRecords;
            m_header.setAdditionalCount((quint16) m_additionalRecords.length());
        }
    }
}