//
// Created by robin on 1/10/18.
//

#include "dns-base/QDnsLabel.h"
#include "dns-base/QDnsResourceRecord.h"

namespace QDns
{
    namespace Base
    {
        QDnsResourceRecord::QDnsResourceRecord()
                : m_isValid(true)
        {
        }

        QDnsResourceRecord QDnsResourceRecord::parse(const QByteArray &packetData, quint32 pos)
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            return parse(packetData, dataStream);
        }

        QDnsResourceRecord QDnsResourceRecord::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            QDnsResourceRecord rr;

            rr.m_name = QDnsLabel::parse(packetData, dataStream);
            if (rr.m_name.isNull() || dataStream.device()->bytesAvailable() < 10)
            {
                rr.m_isValid = false;
            }
            else
            {
                dataStream >> rr.m_type;
                dataStream >> rr.m_class;
                dataStream >> rr.m_ttl;
                quint16 length;
                dataStream >> length;
                if (dataStream.device()->bytesAvailable() < length)
                {
                    rr.m_isValid = false;
                }
                else
                {
                    char data[length];
                    dataStream.readRawData(data, length);
                    rr.m_data.append(data, length);
                }
            }
            return rr;
        }

        QByteArray QDnsResourceRecord::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsResourceRecord::serialize(QDataStream &dataStream) const
        {
            auto label = QDnsLabel::serialize(m_name);
            dataStream.writeRawData(label.constData(), label.length());
            dataStream << m_type;
            dataStream << m_class;
            dataStream << m_ttl;
            dataStream << (quint16) m_data.length();
            dataStream.writeRawData(m_data.constData(), m_data.length());
        }

        bool QDnsResourceRecord::isValid() const
        {
            return m_isValid;
        }

        const QString &QDnsResourceRecord::getName() const
        {
            return m_name;
        }

        void QDnsResourceRecord::setName(const QString &name)
        {
            m_name = name;
        }

        quint16 QDnsResourceRecord::getType() const
        {
            return m_type;
        }

        void QDnsResourceRecord::setType(quint16 type)
        {
            m_type = type;
        }

        quint16 QDnsResourceRecord::getClass() const
        {
            return m_class;
        }

        void QDnsResourceRecord::setClass(quint16 aClass)
        {
            m_class = aClass;
        }

        quint32 QDnsResourceRecord::getTtl() const
        {
            return m_ttl;
        }

        void QDnsResourceRecord::setTtl(quint32 ttl)
        {
            m_ttl = ttl;
        }

        const QByteArray &QDnsResourceRecord::getData() const
        {
            return m_data;
        }

        void QDnsResourceRecord::setData(const QByteArray &data)
        {
            m_data = data;
        }
    }
}